@@ -300,8 +300,8 @@ module Generate_bin_size = struct
               let size_args = $size_args$ in
               Pervasives.(+) size_args 4
           >>
-      | <:ctyp< [= $row_fields$ ] >> | <:ctyp< [> $row_fields$ ] >>
-      | <:ctyp< [< $row_fields$ ] >> -> loop row_fields
+      | <:ctyp< [< $row_fields$ ] >> | <:ctyp< [> $row_fields$ ] >>
+      | <:ctyp< [= $row_fields$ ] >> -> loop row_fields
       | <:ctyp< $tp1$ $tp2$ >> ->
           let id_path = Gen.get_appl_path _loc tp1 in
           let call = bin_size_appl_fun full_type_name _loc tp1 tp2 in
@@ -583,8 +583,8 @@ module Generate_bin_write = struct
               in
               $write_args$
           >>
-      | <:ctyp< [= $row_fields$ ] >> | <:ctyp< [> $row_fields$ ] >>
-      | <:ctyp< [< $row_fields$ ] >> -> loop row_fields
+      | <:ctyp< [< $row_fields$ ] >> | <:ctyp< [> $row_fields$ ] >>
+      | <:ctyp< [= $row_fields$ ] >> -> loop row_fields
       | <:ctyp< $tp1$ $tp2$ >> ->
           let id_path = Gen.get_appl_path _loc tp1 in
           let call = bin_write_appl_fun full_type_name _loc tp1 tp2 in
@@ -1157,7 +1157,9 @@ module Generate_bin_read = struct
 
   let bin_read_td _loc type_name tps rhs =
     let full_type_name = sprintf "%s.%s" (get_conv_path ()) type_name in
-    let coll_args tp param = <:ctyp< $tp$ $param$ >> in
+    let coll_args tp param =
+      <:ctyp< $tp$ $Gen.drop_variance_annotations _loc param$ >>
+    in
     let full_type = List.fold_left coll_args <:ctyp< $lid:type_name$ >> tps in
     let is_alias_ref = ref false in
     let handle_alias _loc tp =
