@@ -533,3 +533,33 @@ external bin_read_int64_bits :
 
 let bin_read_int64_bits buf ~pos_ref =
   rewrite_c_error bin_read_int64_bits buf ~pos_ref
+
+external bin_read_network16_int :
+  buf -> pos_ref : pos_ref -> int = "ml_read_network16_int_stub"
+
+let bin_read_network16_int buf ~pos_ref =
+  rewrite_c_error bin_read_network16_int buf ~pos_ref
+
+external bin_read_network32_int :
+  buf -> pos_ref : pos_ref -> int = "ml_read_network32_int_stub"
+
+let bin_read_network32_int buf ~pos_ref =
+  rewrite_c_error bin_read_network32_int buf ~pos_ref
+
+external bin_read_network32_int32 :
+  buf -> pos_ref : pos_ref -> int32 = "ml_read_network32_int32_stub"
+
+let bin_read_network32_int32 buf ~pos_ref =
+  rewrite_c_error bin_read_network32_int32 buf ~pos_ref
+
+external bin_read_network64_int :
+  buf -> pos_ref : pos_ref -> int = "ml_read_network64_int_stub"
+
+let bin_read_network64_int buf ~pos_ref =
+  rewrite_c_error bin_read_network64_int buf ~pos_ref
+
+external bin_read_network64_int64 :
+  buf -> pos_ref : pos_ref -> int64 = "ml_read_network64_int64_stub"
+
+let bin_read_network64_int64 buf ~pos_ref =
+  rewrite_c_error bin_read_network64_int64 buf ~pos_ref
