@@ -103,7 +103,7 @@ static inline void raise_Read_error(int loc, unsigned long pos)
   MK_GEN_SAFE_READ(PREF##nat0_, SIZE, unsigned TYPE, LEN, CHECK)
 
 #define MK_SAFE_NAT0_READ(SIZE, TYPE, LEN, CHECK) \
-  MK_GEN_SAFE_NAT0_READ(, SIZE, TYPE, LEN, CHECK)
+  MK_GEN_SAFE_READ(nat0_, SIZE, unsigned TYPE, LEN, CHECK)
 
 #define MK_ML_READER(NAME) \
   CAMLprim value ml_read_##NAME##_stub(value v_buf, value v_pos_ref) \
@@ -131,17 +131,17 @@ MK_GEN_SAFE_READ(neg_int, 8, char, 1,
     raise_Error(READ_ERROR_NEG_INT8);
   })
 
-MK_SAFE_READ(16, short, 2,)
+MK_SAFE_READ(16, short, 2, {})
 
 #ifdef ARCH_SIXTYFOUR
-  MK_SAFE_READ(32, int, 4,)
+  MK_SAFE_READ(32, int, 4, {})
 #else
   MK_GEN_SAFE_READ(int, 32, int, 4,
     if (unlikely(n < -0x40000000l || n > 0x3FFFFFFFl)) {
       *sptr_ptr = sptr - 1;
       raise_Error(READ_ERROR_INT_OVERFLOW);
     })
-  MK_GEN_SAFE_READ(nocheck_int, 32, int, 4,)
+  MK_GEN_SAFE_READ(nocheck_int, 32, int, 4, {})
 #endif
 
 #ifdef ARCH_SIXTYFOUR
@@ -150,7 +150,7 @@ MK_SAFE_READ(16, short, 2,)
       *sptr_ptr = sptr - 1;
       raise_Error(READ_ERROR_INT_OVERFLOW);
     })
-  MK_GEN_SAFE_READ(nocheck_int, 64, long, 8,)
+  MK_GEN_SAFE_READ(nocheck_int, 64, long, 8, {})
 #endif
 
 static inline long read_int(char **sptr_ptr, char *eptr)
@@ -179,17 +179,17 @@ CAMLprim value read_int_stub(char **sptr_ptr, char *eptr)
 
 /* Non-negative OCaml integers */
 
-MK_SAFE_NAT0_READ(16, short, 2,)
+MK_SAFE_NAT0_READ(16, short, 2, {})
 
 #ifdef ARCH_SIXTYFOUR
-  MK_SAFE_NAT0_READ(32, int, 4,)
+  MK_SAFE_NAT0_READ(32, int, 4, {})
 #else
   MK_SAFE_NAT0_READ(32, int, 4,
     if (unlikely(n > 0x3FFFFFFFl)) {
       *sptr_ptr = sptr - 1;
       raise_Error(READ_ERROR_NAT0_OVERFLOW);
     })
-  MK_GEN_SAFE_NAT0_READ(nocheck, 32, int, 4,)
+  MK_GEN_SAFE_NAT0_READ(nocheck, 32, int, 4, {})
 #endif
 
 #ifdef ARCH_SIXTYFOUR
@@ -198,7 +198,7 @@ MK_SAFE_NAT0_READ(16, short, 2,)
       *sptr_ptr = sptr - 1;
       raise_Error(READ_ERROR_NAT0_OVERFLOW);
     })
-  MK_GEN_SAFE_NAT0_READ(nocheck, 64, long, 8,)
+  MK_GEN_SAFE_NAT0_READ(nocheck, 64, long, 8, {})
 #endif
 
 static inline unsigned long read_nat0(char **sptr_ptr, char *eptr)
@@ -500,6 +500,21 @@ CAMLprim value ml_read_variant_tag_stub(value v_buf, value v_pos_ref)
 }
 
 
+/* Reading raw strings */
+
+CAMLprim inline value read_raw_string_stub(
+  char **sptr_ptr, char *eptr, value v_str, value v_pos, value v_len)
+{
+  size_t pos = (size_t) Long_val(v_pos), len = (size_t) Long_val(v_len);
+  char *sptr = *sptr_ptr;
+  char *next = sptr + len;
+  if (unlikely(next > eptr)) caml_raise_constant(*v_bin_prot_exc_Buffer_short);
+  *sptr_ptr = next;
+  memcpy(String_val(v_str) + pos, sptr, len);
+  return Val_unit;
+}
+
+
 /* Reading bigarrays */
 
 #define MK_BA1_READER(NAME, TYPE, TFLAG) \
@@ -626,8 +641,8 @@ CAMLprim inline value read_int64_bits_stub(char **sptr_ptr, char *eptr)
 #else
   {
     unsigned int *uisptr = (unsigned int *) sptr;
-    unsigned lower = *uisptr++;
-    unsigned upper = *uisptr;
+    unsigned int lower = *uisptr++;
+    unsigned int upper = *uisptr;
     n = I64_or(I64_lsl(I64_of_int32(upper), 32), I64_of_int32(lower));
   }
 #endif
@@ -636,3 +651,116 @@ CAMLprim inline value read_int64_bits_stub(char **sptr_ptr, char *eptr)
   return (value) v_res;
 }
 MK_ML_READER(int64_bits)
+
+CAMLprim inline value read_network16_int_stub(char **sptr_ptr, char *eptr)
+{
+  char *sptr = *sptr_ptr;
+  char *next = sptr + 2;
+  if (unlikely(next > eptr)) caml_raise_constant(*v_bin_prot_exc_Buffer_short);
+  *sptr_ptr = next;
+  return (value) Val_int((uint16_t) ntohs(*((uint16_t *) sptr)));
+}
+MK_ML_READER(network16_int)
+
+CAMLprim inline value read_network32_int_stub(char **sptr_ptr, char *eptr)
+{
+  char *sptr = *sptr_ptr;
+  char *next = sptr + 4;
+  int n;
+  if (unlikely(next > eptr)) caml_raise_constant(*v_bin_prot_exc_Buffer_short);
+  n = (int) ntohl(*(uint32_t *) sptr);
+#ifndef ARCH_SIXTYFOUR
+  if (unlikely(n < -0x40000000l || n > 0x3FFFFFFFl))
+    raise_Error(READ_ERROR_INT_OVERFLOW);
+  *sptr_ptr = next;
+  return (value) Val_int((int) ntohl(*((uint32_t *) sptr)));
+#else
+  *sptr_ptr = next;
+  return (value) Val_int((uint32_t) ntohl(*((uint32_t *) sptr)));
+#endif
+}
+MK_ML_READER(network32_int)
+
+CAMLprim inline value read_network32_int32_stub(char **sptr_ptr, char *eptr)
+{
+  char *sptr = *sptr_ptr;
+  char *next = sptr + 4;
+  if (unlikely(next > eptr)) caml_raise_constant(*v_bin_prot_exc_Buffer_short);
+  *sptr_ptr = next;
+  return (value) caml_copy_int32((int) ntohl(*((uint32_t *) sptr)));
+}
+MK_ML_READER(network32_int32)
+
+CAMLprim inline value read_network64_int_stub(char **sptr_ptr, char *eptr)
+{
+  char *sptr = *sptr_ptr;
+  char *next = sptr + 8;
+  long n;
+  if (unlikely(next > eptr)) caml_raise_constant(*v_bin_prot_exc_Buffer_short);
+#ifdef ARCH_SIXTYFOUR
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+  n = (long) bswap_64(*((uint64_t *) sptr));
+#elif __BYTE_ORDER == __BIG_ENDIAN
+  n = *((long *) sptr);
+#else
+#error "unsupported endianness"
+#endif
+  if (unlikely(n < -0x4000000000000000L || n > 0x3FFFFFFFFFFFFFFFL))
+    raise_Error(READ_ERROR_INT_OVERFLOW);
+#else /* 32bit */
+  /* Read the upper 32 bits first.  They must all be zero, otherwise we
+     consider this an overflow.  On 32bit platforms the integer must
+     fit completely into one word. */
+  n = *((long *) sptr);
+  if (n != 0) raise_Error(READ_ERROR_INT_OVERFLOW);
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+  n = (long) bswap_32(*(((uint32_t *) sptr) + 1));
+#elif __BYTE_ORDER == __BIG_ENDIAN
+  n = *(((long *) sptr) + 1);
+#else
+#error "unsupported endianness"
+#endif
+  if (unlikely(n < -0x40000000l || n > 0x3FFFFFFFl))
+    raise_Error(READ_ERROR_INT_OVERFLOW);
+#endif
+  *sptr_ptr = next;
+  return (value) Val_long(n);
+}
+MK_ML_READER(network64_int)
+
+CAMLprim inline value read_network64_int64_stub(char **sptr_ptr, char *eptr)
+{
+  char *sptr = *sptr_ptr;
+  char *next = sptr + 8;
+  int64 n;
+  if (unlikely(next > eptr)) caml_raise_constant(*v_bin_prot_exc_Buffer_short);
+  *sptr_ptr = next;
+#ifdef ARCH_SIXTYFOUR
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+  n = (long) bswap_64(*((uint64_t *) sptr));
+#elif __BYTE_ORDER == __BIG_ENDIAN
+  n = *((long *) sptr);
+#else
+#error "unsupported endianness"
+#endif
+  return (value) caml_copy_int64(n);
+#else /* 32bit */
+  {
+    uint32_t *uisptr = (uint32_t *) sptr;
+    uint32_t upper = *uisptr++;
+    uint32_t lower = *uisptr;
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+    n =
+      I64_or(
+        I64_lsl(I64_of_int32(bswap_32(upper)), 32),
+        I64_of_int32(bswap_32(lower)));
+#elif __BYTE_ORDER == __BIG_ENDIAN
+    n = I64_or(I64_lsl(I64_of_int32(upper), 32), I64_of_int32(lower));
+#else
+#error "unsupported endianness"
+#endif
+    return (value) caml_copy_int64(n);
+  }
+#endif
+}
+MK_ML_READER(network64_int64)
