@@ -258,7 +258,7 @@ module type Make_iterable_binable_spec = sig
   type acc
 
   val module_name : string option
-  val get_length : t -> int
+  val length : t -> int
   val iter : f : (el -> unit) -> t -> unit
   val init : int -> acc
   val insert : acc -> el -> int -> acc
@@ -287,12 +287,12 @@ module Make_iterable_binable (S : Make_iterable_binable_spec) = struct
     iter t ~f:(fun el ->
       size_ref := !size_ref + bin_size_el el;
       incr cnt_ref);
-    let len = get_length t in
+    let len = length t in
     if !cnt_ref = len then bin_size_nat0 (Nat0.unsafe_of_int len) + !size_ref
     else raise_concurrent_modification "bin_size_t"
 
   let bin_write_t_ sptr eptr t =
-    let len = get_length t in
+    let len = length t in
     let plen = Nat0.unsafe_of_int len in
     let els_sptr = Unsafe_write_c.bin_write_nat0 sptr eptr plen in
     let cnt_ref = ref 0 in
@@ -301,7 +301,7 @@ module Make_iterable_binable (S : Make_iterable_binable_spec) = struct
       cur_ref := bin_write_el_ !cur_ref eptr el;
       incr cnt_ref);
     if !cnt_ref = len then !cur_ref
-    else raise_concurrent_modification "bin_write_t"
+    else raise_concurrent_modification "bin_write_t_"
 
   let bin_write_t buf ~pos t =
     let start, sptr, eptr = Write_c.unsafe_get_init buf ~pos in
@@ -365,7 +365,7 @@ module type Make_iterable_binable1_spec = sig
   type 'a acc
 
   val module_name : string option
-  val get_length : 'a t -> int
+  val length : 'a t -> int
   val iter : f : ('a el -> unit) -> 'a t -> unit
   val init : int -> 'a acc
   val insert : 'a acc -> 'a el -> int -> 'a acc
@@ -394,12 +394,12 @@ module Make_iterable_binable1 (S : Make_iterable_binable1_spec) = struct
     iter t ~f:(fun el ->
       size_ref := !size_ref + bin_size_el bin_size_a el;
       incr cnt_ref);
-    let len = get_length t in
+    let len = length t in
     if !cnt_ref = len then bin_size_nat0 (Nat0.unsafe_of_int len) + !size_ref
     else raise_concurrent_modification "bin_size_t"
 
   let bin_write_t_ bin_write_a_ sptr eptr t =
-    let len = get_length t in
+    let len = length t in
     let plen = Nat0.unsafe_of_int len in
     let els_sptr = Unsafe_write_c.bin_write_nat0 sptr eptr plen in
     let cnt_ref = ref 0 in
@@ -408,7 +408,7 @@ module Make_iterable_binable1 (S : Make_iterable_binable1_spec) = struct
       cur_ref := bin_write_el_ bin_write_a_ !cur_ref eptr el;
       incr cnt_ref);
     if !cnt_ref = len then !cur_ref
-    else raise_concurrent_modification "bin_write_t"
+    else raise_concurrent_modification "bin_write_t_"
 
   let bin_write_t bin_write_a_ buf ~pos t =
     let start, sptr, eptr = Write_c.unsafe_get_init buf ~pos in
@@ -478,7 +478,7 @@ module type Make_iterable_binable2_spec = sig
   type ('a, 'b) acc
 
   val module_name : string option
-  val get_length : ('a, 'b) t -> int
+  val length : ('a, 'b) t -> int
   val iter : f : (('a, 'b) el -> unit) -> ('a, 'b) t -> unit
   val init : int -> ('a, 'b) acc
   val insert : ('a, 'b) acc -> ('a, 'b) el -> int -> ('a, 'b) acc
@@ -507,12 +507,12 @@ module Make_iterable_binable2 (S : Make_iterable_binable2_spec) = struct
     iter t ~f:(fun el ->
       size_ref := !size_ref + bin_size_el bin_size_a bin_size_b el;
       incr cnt_ref);
-    let len = get_length t in
+    let len = length t in
     if !cnt_ref = len then bin_size_nat0 (Nat0.unsafe_of_int len) + !size_ref
     else raise_concurrent_modification "bin_size_t"
 
   let bin_write_t_ bin_write_a_ bin_write_b_ sptr eptr t =
-    let len = get_length t in
+    let len = length t in
     let plen = Nat0.unsafe_of_int len in
     let els_sptr = Unsafe_write_c.bin_write_nat0 sptr eptr plen in
     let cnt_ref = ref 0 in
@@ -521,7 +521,7 @@ module Make_iterable_binable2 (S : Make_iterable_binable2_spec) = struct
       cur_ref := bin_write_el_ bin_write_a_ bin_write_b_ !cur_ref eptr el;
       incr cnt_ref);
     if !cnt_ref = len then !cur_ref
-    else raise_concurrent_modification "bin_write_t"
+    else raise_concurrent_modification "bin_write_t_"
 
   let bin_write_t bin_write_a_ bin_write_b_ buf ~pos t =
     let start, sptr, eptr = Write_c.unsafe_get_init buf ~pos in
